import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

from numpy import *
from scipy import *
from qutip import *

# Define atomic states
# Use ordering from paper
state2 = basis(3,0)
excited = basis(3,1)
ground = basis(3,2)

# Set where to truncate Fock state for cavity
N = 30

# Create the atomic operators needed for the Hamiltonian
# |g><e|
sigma_ge = tensor(qeye(N), ground * excited.dag())
# |e><2|
sigma_e2 = tensor(qeye(N), excited * state2.dag())
# |g><2|
sigma_g2 = tensor(qeye(N), ground * state2.dag())

# Create the photon operator
a = tensor(destroy(N), qeye(3))
ada = tensor(num(N), qeye(3))

# Define collapse operators
c_ops = []
# Cavity decay rate
kappa = 0.04
c_ops.append(sqrt(kappa) * a)

# Atomic decay rate
gamma1 = 1.3164239028e-6
gamma2 = 1e7 * gamma1
c_ops.append(sqrt(gamma1) * sigma_ge)
c_ops.append(sqrt(gamma2) * sigma_e2)

g = 0.2
Omega = 10 * kappa

# Hamiltonian
H = g * (sigma_ge.dag() * a + a.dag() * sigma_ge) + 0.5 * Omega * (sigma_g2 + sigma_g2.dag())

taus = linspace(0, 2e2, 2e2)

fig, ax = plt.subplots(1, 1)

g2 = coherence_function_g2(H, taus, c_ops, a)

ax.plot(taus, real(g2), label='g='+str(g)+', amplitude='+str(real(max(g2)-min(g2))), lw=2)
# ax.plot(taus, real(g2), label='g=0.2, rp='+str(rp), lw=2)

ax.legend(loc=0)
ax.set_xlabel(r'$\tau$')
ax.set_ylabel(r'$g^{(2)}(\tau)$')
plt.savefig('g2tau.png')